#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "../../inc/lexos2.h"
#include "../../inc/pci.h"

#include "config.h"

#include "nvtypes.h"

#include "Nv04.h"
#include "./nv04regs/nv04hwregs.h"
#include "nv4ref.h"
#include "instanceMem.h"

_DriverInfo *nvdriverinfo    = NULL ;   
_OsCalls    *os              = NULL ;
_TimingData td[0x10] ;
int timingMax = 0 ;

#include "alloc.c"
#include "debug.c"

#include "dac.c"
#include "nv04DeviceInit.c"
#include "gr.c"
#include "readtiming.c"
#include "grf.c" 
#include "fifo.c"
#include "GrObj.c"
#include "dma.c"

  
#include "dma2pio.c" 


#include "3dtest.c"
/* #include "3dDmaTest.c" */
#include "DmaTest.c"
#include "user.c"




#define LEXOS_Use_CmdChannel 0 

_DriverInfo * NV04_Get_DriverInfoAddr() ;




void Nv04_DriverCommandChannel(_CommandChannelData *cmdch)
{
  _Nv04Device     *nv       ;
  _Nv04_VideoUser *testuser ;
  PciSubDevice *nvpci = (void *) cmdch->initPrams ;
  _MemInstance * praminFirstInst ;
  _MemInstance * LocalMemFirstInst ;
  
  unsigned long i ;
  char *memFileName = "/dev/mem" ;

  nvdriverinfo = NV04_Get_DriverInfoAddr() ;
  os = nvdriverinfo->cmdch.OsCall ;


  nv              = os->NewCildObjn(nvdriverinfo->root , sizeof(_Nv04Device) , "NV04-06 Driver" ) ;
  nv->info.Fb     = os->NewCildObjn(nv , sizeof(_MemHeader)                  , "nvFrameBuffer"  ) ;
  nv->info.Pramin = os->NewCildObjn(nv , sizeof(_MemHeader)                  , "nvPRamIn"       ) ;
  nv->Users       = os->NewCildObjn(nv , sizeof(_Nv04_VideoUser)             , "nvUsers"        ) ;


  nv->memFile = open( memFileName , O_RDWR ) ;
  if ( nv->memFile == -1 ) { printf("error open mem %s %d %s\n", memFileName , errno , strerror(errno)) ; return ; }

  nv->info.Regs.Base     = 0x00000000 ;
  nv->info.Regs.Limit    = 0x00ffffff ;
  nv->info.Regs.PhysAddr = nvpci->PhysAdds[0] ;
  nv->info.Regs.KernAddr = os->MapPhys( nv->info.Regs.PhysAddr , (nv->info.Regs.Limit + 1)) ;
  nv->info.Regs.UserAddr = nv->info.Regs.KernAddr ;
  
  nv->Regs = (void *) nv->info.Regs.UserAddr ;

  nv->ch   = (void *) nv->Regs + ( 0 ? NV_CHAN0 ) ; 


  nv->info.Pramin->mem.Base      = 0x00000000 ;
  nv->info.Pramin->mem.Limit     = 0x01ffffff ;
  nv->info.Pramin->mem.PhysAddr  = nvpci->PhysAdds[0] + ( 0 ? NV04_PRAMIN ) ;
  nv->info.Pramin->mem.KernAddr  = nv->Regs           + ( 0 ? NV04_PRAMIN ) ;
  nv->info.Pramin->mem.UserAddr  = nv->info.Pramin->mem.KernAddr  ;
  nv->info.Pramin->Free          = 0x0005c000 ;

  praminFirstInst = os->NewCildObjn( nv->info.Pramin , sizeof(_MemInstance)    , "praminFree" ) ; 
  praminFirstInst->base     = 0x00000000 ;
  praminFirstInst->size     = 0x0005c000 ;          /* this is a guest */
  praminFirstInst->flags    = LEXOS_MEM_INSTANCE_FREE ;
 

  nv->info.Fb->mem.Base      = 0x00000000 ;
  nv->info.Fb->mem.Limit     = Nv04_FB_RAM_SIZE - 1 ;
  nv->info.Fb->mem.PhysAddr  = nvpci->PhysAdds[1] ;
  nv->info.Fb->mem.KernAddr  = os->MapPhys( nv->info.Fb->mem.PhysAddr , (nv->info.Fb->mem.Limit + 1)) ;
  nv->info.Fb->mem.UserAddr  = nv->info.Fb->mem.KernAddr  ; 
  nv->info.Fb->MemSize       = nv->info.Fb->mem.Limit + 1 ;
  nv->info.Fb->Free          = nv->info.Fb->mem.Limit     ;

  nv->Fb = (void *) nv->info.Fb->mem.UserAddr ;

  LocalMemFirstInst = os->NewCildObjn( nv->info.Fb , sizeof(_MemInstance)    , "FbFree" ) ; 
  LocalMemFirstInst->base     = 0x00000000 ;
  LocalMemFirstInst->size     = 0x01ffffff ;                /* 32 MB */
  LocalMemFirstInst->flags    = LEXOS_MEM_INSTANCE_FREE ;


  printf("Nv04: Regs %8.8lX ( %8.8lX )  Pramin %8.8lX ( %8.8lX )  Fb %8.8lX ( %8.8lX )\n",nv->info.Regs.PhysAddr , (unsigned long)  nv->info.Regs.UserAddr ,nv->info.Pramin->mem.PhysAddr , (unsigned long)  nv->info.Pramin->mem.UserAddr  ,nv->info.Fb->mem.PhysAddr , (unsigned long)  nv->info.Fb->mem.UserAddr ) ;

  getchar() ;

  timingMax = Nv04_loadtimingdata() ;
  if (timingMax <= 0) { printf("no time res found\n") ; return ; }
  
  Nv04_DeviceInit(nv) ;

  setreg(NV04_PBUS_PCI_NV_1, (getreg(NV04_PBUS_PCI_NV_1) & 0x000000fe) | 0x00000004 ) ; /* io access off & bus master on */ 
  Nv04_SetupDma( nv ) ;
  
  
  if(1)
    {
      setreg( NV_PMC_INTR_EN_0 , 0x00000000 ) ;
      setreg( NV_PMC_ENABLE    , 0xFFFF00FF ) ;
      setreg( NV_PMC_ENABLE    , 0xFFFFFFFF ) ;
      
      setreg( NV_PTIMER_NUMERATOR   , 0x00000008 ) ;
      setreg( NV_PTIMER_DENOMINATOR , 0x00000003 ) ;
      setreg( NV_PTIMER_INTR_EN_0   , 0x00000000 ) ;
      setreg( NV_PTIMER_INTR_0      , 0xFFFFFFFF ) ;
    }

      
      /*  Nv04_mcInit(nv) ; */
      
      testuser = os->NewCildObjn( nv->Users , sizeof(_Nv04_VideoUser)     , "testuser" ) ;
      printf("testuser = %8.8lX\n", ( unsigned long ) testuser) ;


      Nv04_dacInitDac(nv) ;

      os->dumpobjlist((void *) nvdriverinfo->root) ;
      for ( i = 0 ; i < 0x080000 ; i++ ) nv->Fb[i] = i ;

      printf("ready to test user\n") ; getchar() ;

      os->dumpobjlist((void *) nvdriverinfo->root) ;

      Nv04_FifoInit( nv ) ;    
      dumpRegs( nv , fifoSetupRegsTable ) ;

      clrdma(nv ) ;
      Nv04_grInit( nv ) ;


      if(1) TestUser(nv) ;

}

_DriverInfo  Nv04_DriverInfo = 
{
  DriverClass_Video3D ,
  "Nv04 - 05 Driver for linux 0.0.1",
  NULL,
  0,
  LEXOS_DRIVER_STATUS_NOT_LOADED ,
  {
    NULL , 
    NULL ,
    Nv04_DriverCommandChannel
  } , 
  
  devs,
  NULL
} ;

_DriverInfo * NV04_Get_DriverInfoAddr() { return(&Nv04_DriverInfo) ; }

