


typedef struct {
  NvU8 index;
  NvU8 data;
} dacreg_t;

typedef struct { 
  NvU8 ClockMode; 
  NvU8 MapMask; 
  NvU8 FontAddr; 
  NvU8 MemMode;
} seq_t; 

typedef struct {
  dacreg_t presetRowScan; 
  dacreg_t cursorS; 
  dacreg_t cursorE; 
  dacreg_t regenSHigh; 
  dacreg_t regenSLow; 
  dacreg_t cursorPosHigh; 
  dacreg_t cursorPosLow; 
  dacreg_t uLineRow; 
  dacreg_t mode; 
  dacreg_t lineCompare; 
} crtc_t;

typedef struct {
  NvU8 SetReset;
  NvU8 EnableSetReset;
  NvU8 ColorCmp;
  NvU8 ROP;
  NvU8 ReadMap;
  NvU8 Mode;
  NvU8 Misc;
  NvU8 CDC;
  NvU8 BitMask;
} gr_t; 



typedef struct {
  seq_t   seq_regs; 
  crtc_t crtc_regs; 
  gr_t     gr_regs; 
} dacinit_tbl_t; 



dacinit_tbl_t dacinit_tbl = {
  {0x01, 0x0f, 0x00, 0x0e},                                                /* just the std vag Sequencer Registers           */
  
  {{ NV04_CIO_CR_RSAL_INDEX     ,0x00}, 
   { NV04_CIO_CR_CURS_ST_INDEX  ,0x00}, 
   { NV04_CIO_CR_CURS_END_INDEX ,0x00}, 
   { NV04_CIO_CR_SA_HI_INDEX    ,0x00}, 
   { NV04_CIO_CR_SA_LO_INDEX    ,0x00},
   { NV04_CIO_CR_TCOFF_HI_INDEX ,0x00}, 
   { NV04_CIO_CR_TCOFF_LO_INDEX ,0x00}, 
   { NV04_CIO_CR_ULINE_INDEX    ,0x00}, 
   { NV04_CIO_CR_MODE_INDEX     ,0xe3}, 
   { NV04_CIO_CR_LCOMP_INDEX    ,0xff}},                                   /* just the std vga CRT Registers                 */
 
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x05, 0x0f, 0xff}                   /* just the std vga Graphics Controller Registers */
};

NvU8 atc_tbl[20] = {
  0x00, 0x01, 0x02, 0x03, 0x04, 0x05,
  0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b,
  0x0c, 0x0D, 0x0e, 0x0f, 0x41, 0x00,
  0x0f, 0x00,                                                              /* just the std vga Attribute Registers  */
};

typedef struct {
  NvU16 Freq; 
  NvU8 LowWaterMark64; 
  NvU8 BurstSize64; 
  NvU8 LowWaterMark128; 
  NvU8 BurstSize128; 
} fifo_t;

fifo_t DACFifoTable[] =
{
  { 24000, 0x10, NV04_CIO_CRE_FF_BURST_128 , 0x10, NV04_CIO_CRE_FF_BURST_128 },
  { 27000, 0x11, NV04_CIO_CRE_FF_BURST_128 , 0x11, NV04_CIO_CRE_FF_BURST_128 },
  { 28000, 0x12, NV04_CIO_CRE_FF_BURST_128 , 0x12, NV04_CIO_CRE_FF_BURST_128 },
  { 29000, 0x13, NV04_CIO_CRE_FF_BURST_128 , 0x13, NV04_CIO_CRE_FF_BURST_128 },
  { 32000, 0x14, NV04_CIO_CRE_FF_BURST_128 , 0x14, NV04_CIO_CRE_FF_BURST_128 },
  { 33000, 0x15, NV04_CIO_CRE_FF_BURST_128 , 0x15, NV04_CIO_CRE_FF_BURST_128 },
  { 36000, 0x16, NV04_CIO_CRE_FF_BURST_128 , 0x16, NV04_CIO_CRE_FF_BURST_128 },
  { 38000, 0x17, NV04_CIO_CRE_FF_BURST_128 , 0x17, NV04_CIO_CRE_FF_BURST_128 },
  { 39000, 0x18, NV04_CIO_CRE_FF_BURST_128 , 0x18, NV04_CIO_CRE_FF_BURST_128 },
  { 42000, 0x1A, NV04_CIO_CRE_FF_BURST_128 , 0x1A, NV04_CIO_CRE_FF_BURST_128 },
  { 50000, 0x22, NV04_CIO_CRE_FF_BURST_64  , 0x22, NV04_CIO_CRE_FF_BURST_64  },
  { 65535, 0x25, NV04_CIO_CRE_FF_BURST_32  , 0x25, NV04_CIO_CRE_FF_BURST_32  }         /* fifo LowWaterMark and BurstSize for given freq */
}; 

void WriteIndexed(_Nv04Device *nv , NvU32 addr, NvU16 datum)
{ 
 
  setreg8( addr     , ( datum        & 0xff )) ;
  setreg8( addr + 1 , ( (datum >> 8) & 0xff )) ;
} 

NvU16 ReadIndexed(_Nv04Device *nv , NvU32 addr, NvU8 reg)
{ 

  NvU16 datum;
  setreg8(( addr ) , ( reg ))  ;
  datum = (getreg8(addr+1 ) << 8) & 0xff00 ;
  datum |= reg;
 return datum;
} 

void setindexreg(_Nv04Device * nv , unsigned long addr , unsigned long index , unsigned long data ) 
{
  i386_setreg8(nv->Regs , addr , index ) ; i386_setreg8( nv->Regs , addr + 1 , data ) ;
}

void modindexreg(_Nv04Device * nv , unsigned long addr , unsigned long index , unsigned long AndData , unsigned long OrData ) 
{
  i386_setreg8(nv->Regs , addr , index ) ; i386_setreg8( nv->Regs , addr + 1 , (i386_getreg8( nv->Regs ,addr + 1) & AndData) | OrData) ; 
}

unsigned char getindexreg(_Nv04Device * nv , unsigned long addr , unsigned long index ) 
{
  i386_setreg8(nv->Regs , addr , index ) ; return( i386_getreg8( nv->Regs ,addr + 1)) ;
}




void LoadIndexRange(_Nv04Device *nv , NvU32 addr , NvU8 index , NvU8 count , NvU8 *regs)
{

  int i;
  for (i = 0; i < count; i++) 
    {
      setreg8( addr , (index ) & 0xff ) ; setreg8( addr + 1 , ( ( *regs ) & 0xff ) ) ;
      index++;
      regs++;
    }
}

void Nv04_UnlockDac(_Nv04Device * nv) 
{
  unsigned long wv ;
  setindexreg(   nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_SR_LOCK_INDEX , NV04_CIO_SR_UNLOCK_RW_VALUE ) ;
  
  wv = getindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x21 ) ; setindexreg( nv , NV04_PRMCIO_CRX__COLOR , wv , 0xc8 ) ;

}

void Nv04_LockDac(_Nv04Device * nv) 
{
  setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_SR_LOCK_INDEX , NV04_CIO_SR_LOCK_VALUE ) ;                     /* lock */
}


void Nv04_fixLowWaterMark(_Nv04Device * nv , unsigned long clock ) 
{
  unsigned long i , wv , lwv ;
  
  wv = getindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_PIXEL_INDEX ) ;
  
  clock = clock / 10000 ;
  
  if ( wv & 3 ) 
    { wv-- ; clock <<= wv ;
    for ( i = 0 ; clock > DACFifoTable[i].Freq ; i++ ) ;
    
    lwv = getfld( NV_PFB_BOOT_0 , NV_PFB_BOOT_0_RAM_WIDTH_128 ) ;
    
    if ( lwv == NV_PFB_BOOT_0_RAM_WIDTH_128_ON ) 
      {
	lwv = DACFifoTable[i].LowWaterMark128 ; 
	i   = DACFifoTable[i].BurstSize128    ;
      }
    else
      {
	lwv = DACFifoTable[i].LowWaterMark64 ; 
	i   = DACFifoTable[i].BurstSize64    ;
      }
    setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_FFLWM__INDEX , lwv ) ;
    setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_FF_INDEX     , i  ) ;
    }
}








NvU32 Nv04_CalcPLL(_Nv04Device * nv , unsigned long  pclock) 
{

  unsigned long data ;
  NvU16 crystalFreq;
  NvU32 vclk;
  NvU16 vclkClosest;
  NvU16 deltaOld;
  NvU16 bestM;
  NvU16 bestP;
  NvU16 bestN;
  NvU16 lowM;
  NvU16 highM;
  NvU32 hiVclk1, hiVclk2;
  NvU16 powerP;
  NvU16 m;
  NvU16 n;
  NvU16 i, j;
  NvU32 lwv;
  NvU32 lwv2;
  NvU16 lwvs, vclks;
  NvU16 t;
  NvU32 fMaxVco; 
  NvU16  aclock ;
  NvU16 *clock  ;
  


  aclock = ( pclock / 10000 ) ;

  clock = (void *) &aclock ;

  vclkClosest = 0 ;
  bestM       = 0 ;
  bestN       = 0 ;
  bestP       = 0 ;

  
  


  vclk = (NvU32)(*clock * 10); 
  deltaOld = 0xffff;
  hiVclk1 = 250000;
  hiVclk2 = 340000;
  if (vclk < 256000)
    fMaxVco = 256000;
  else
   fMaxVco = vclk;


  lwv = getreg( NV04_PEXTDEV_BOOT_0 ) & 0x00000040 ; /*  crystalFreq  */
  
  if (lwv) 
    {
      crystalFreq = 14318 ;
      lowM = 1;
      if (vclk > hiVclk2)
	highM = 2;
      else if (vclk > hiVclk1)
	highM = 6;
      else
	highM = 14;
    } 
  else 
    {
      crystalFreq = 13500 ;
      lowM = 1;
      if (vclk > hiVclk2)
	highM = 2;
      else if (vclk > hiVclk1)
	highM = 6;
      else
	highM = 13;
    }

  for (powerP = 0; powerP <= 4; powerP++) 
    { 
      for (m = lowM; m <= highM; m++) 
	{
	  lwv = vclk << powerP; 
	  if ((lwv >= 128000) && (lwv <= fMaxVco)) 
	    {
	      lwv *= (NvU32)m; 
	      lwv += (crystalFreq >> 1);
	      lwv /= crystalFreq; 
	      n = (NvU16)(lwv & 0xffff);
	      if (n > 0xff) break;
	      lwv *= crystalFreq; 
	      i = 1 << powerP; 
	      i >>= 1; 
	      lwv += i; 
	      lwv >>= powerP; 
	      i = m;
	      j = m;
	      j >>= 1; 
	      lwv += j; 
	      lwv /= m; 
	      lwv2 = lwv; 
	      lwvs = (NvU16)(lwv & 0xffff);
	      vclks = (NvU16)(vclk & 0xffff);
	      if (lwvs < vclks) {
		t = lwvs;
		lwvs = vclks;
		vclks = t;
	      }
	      lwvs -= vclks; 

	      data  = getreg( NV04_PMC_BOOT_0 ) ;
	      if ( ( rdbits( data , NV04_PMC_BOOT_0_ARCHITECTURE   )  > NV04_PMC_BOOT_0_ARCHITECTURE_NV4 ) |
		   ((rdbits( data , NV04_PMC_BOOT_0_ARCHITECTURE   ) == NV04_PMC_BOOT_0_ARCHITECTURE_NV4 ) & 
		    (rdbits( data , NV04_PMC_BOOT_0_MASK_REVISION  ) >= NV04_PMC_BOOT_0_MASK_REVISION_B  ))
		   )
		{     
		  if (vclk > hiVclk1) 
		    {
		      if (((vclk/lwvs) > 200) || (lwvs == 0)) 
			{
			  lwv = powerP;
			  lwv <<= 16;
			  lwv |= ((n << 8) | m);
			  lwv2 /= 10;
			  *clock = ((NvU16)(lwv2 & 0xffff));
			  return lwv;
			}
		    }
		  if (lwvs < deltaOld) 
		    {
		      deltaOld = lwvs;
		      bestP = powerP;
		      bestM = m;
		      bestN = n;
		      lwv2 /= 10;
		      vclkClosest = (NvU16)(lwv2 & 0xffff);
		    }
		} 
	      else 
		{
		  if (lwvs < deltaOld) 
		    {
		      deltaOld = lwvs;
		      bestP = powerP;
		      bestM = m;
		      bestN = n;
		      lwv2 /= 10;
		      vclkClosest = (NvU16)(lwv2 & 0xffff);
		    }
		}
	    }
	}
    }
  lwv = bestP;
  lwv <<= 16;
  lwv |= ((bestN << 8) | (bestM));
  ;
 ;
 *clock = vclkClosest;
    
 printf("Nv04: Nv04_CallPLL %ld  %d  %8.8lX\n",pclock , vclkClosest ,lwv) ;
 return lwv;
 
 
}


void Nv04_dacInitDac( _Nv04Device *nv)
{

  dacreg_t *regs;
  int i ;

    Nv04_UnlockDac(nv) ;

    setindexreg(   nv , NV04_PRMVIO_SRX , NV04_PRMVIO_SR_RESET_INDEX  , 0x01  ) ;
    LoadIndexRange(nv , NV04_PRMVIO_SRX , 1 ,sizeof (seq_t), (NvU8 *)&(dacinit_tbl.seq_regs));
    setindexreg(   nv , NV04_PRMVIO_SRX , NV04_PRMVIO_SR_RESET_INDEX  , 0x03  ) ;
  
    regs = (dacreg_t *)&dacinit_tbl.crtc_regs;
    for (i = 0; i < sizeof (crtc_t) / sizeof (dacreg_t); i++) setindexreg( nv , NV04_PRMCIO_CRX__COLOR , regs[i].index , regs[i].data ) ;	  
     
    LoadIndexRange( nv , NV04_PRMVIO_GRX , 0,sizeof (gr_t), (NvU8 *) &(dacinit_tbl.gr_regs));
   
    Nv04_LockDac(nv) ;

}

void Nv04_dacImageOnOff(_Nv04Device * nv , unsigned long state ) 
{
  Nv04_UnlockDac(nv) ;
  modindexreg(  nv , NV04_PRMVIO_SRX , NV04_PRMVIO_SR_CLOCK_INDEX , ~0x20 , ( (state == 0) << 5)) ; 
  Nv04_LockDac(nv) ;
}



void Nv04_dacSetPixelClock(_Nv04Device * nv , unsigned long hz)
{

  NvU32 mnp;
  NvU16 clock ;

  clock = (NvU16)(hz / 10000);

  if (hz == 0) 
    {
      modreg( NV04_PRAMDAC_PLL_SETUP_CONTROL , NV04_PRAMDAC_PLL_SETUP_CONTROL_PWRDWN , NV04_PRAMDAC_PLL_SETUP_CONTROL_PWRDWN_VPLL ) ;
      return;
    }

  if ( getfld ( NV04_PRAMDAC_PLL_SETUP_CONTROL , NV04_PRAMDAC_PLL_SETUP_CONTROL_PWRDWN ) ==  NV04_PRAMDAC_PLL_SETUP_CONTROL_PWRDWN_VPLL )    
    modreg( NV04_PRAMDAC_PLL_SETUP_CONTROL , NV04_PRAMDAC_PLL_SETUP_CONTROL_PWRDWN , NV04_PRAMDAC_PLL_SETUP_CONTROL_PWRDWN_ON ) ;

  Nv04_fixLowWaterMark( nv , clock); 
  mnp = Nv04_CalcPLL( nv , hz) ; setreg( NV04_PRAMDAC_VPLL_COEFF , mnp ) ;
}

void Nv04_setDacOffset( _Nv04Device *nv , unsigned long pitch)
{
  unsigned long offset = pitch / 8 ;
  modindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_RPC0_INDEX  , 0x1f , ( offset & 0x0700)  >> (  8 - 5)) ;
  modindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_LSR_INDEX   , 0xef , ((offset & 0x0800) >> ( 11 - 5))) ;
  modindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_OFFSET_INDEX , 0x00 , ( offset & 0x00ff              )) ; 
}


int  Nv04_dacEnableImageScan( _Nv04Device * nv)
{


  return(0) ;
}

void Nv04_dacStartImage( _Nv04Device *nv , _Nv04_VideoUser * user )
{
 
 
  NvU32 lrv ;

  Nv04_UnlockDac( nv ) ;
  getchar() ;
  Nv04_setDacOffset(nv , user->BytesPerLine );
  
  lrv = getreg( NV04_PRAMDAC_GENERAL_CONTROL ) ;

  lrv &= ~(modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX     , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX_ON     ) | modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_ALT_MODE , NV04_PRAMDAC_GENERAL_CONTROL_ALT_MODE_SEL )) ;
  lrv |=   modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_VGA_STATE  , NV04_PRAMDAC_GENERAL_CONTROL_VGA_STATE_SEL ) ;
  lrv |=   modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_BPC        , NV04_PRAMDAC_GENERAL_CONTROL_BPC_8BITS     ) ;

  switch(user->ColourDepth) 
    {
    case 8:
      lrv |= modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX     , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX_ON     ) ;

      lrv |= ((0x00000003 )<< ((0? 5:4 ) % 32) ) ;

      setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_PIXEL_INDEX , NV04_CIO_CRE_PIXEL_FORMAT_8BPP ) ;
      break;

    case 15:
      lrv |= (modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX     , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX_ON     )) | modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_ALT_MODE , NV04_PRAMDAC_GENERAL_CONTROL_ALT_MODE_SEL ) ;
     
      setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_PIXEL_INDEX , NV04_CIO_CRE_PIXEL_FORMAT_16BPP ) ;
      break;

    case 16:
      lrv |= modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX_OFF     ) | modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_ALT_MODE , NV04_PRAMDAC_GENERAL_CONTROL_ALT_MODE_NOTSEL ) ;
      setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_PIXEL_INDEX , NV04_CIO_CRE_PIXEL_FORMAT_16BPP ) ; 
      break;
      
    case 24:
    case 32:
      
      lrv |= modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX     , NV04_PRAMDAC_GENERAL_CONTROL_PIXMIX_OFF     ) | modbits( 0x00 , NV04_PRAMDAC_GENERAL_CONTROL_ALT_MODE , NV04_PRAMDAC_GENERAL_CONTROL_ALT_MODE_NOTSEL ) ;  
      setindexreg( nv , NV04_PRMCIO_CRX__COLOR ,  NV04_CIO_CRE_PIXEL_INDEX , NV04_CIO_CRE_PIXEL_FORMAT_32BPP ) ; 
      break;
      
    default:
      
    }
  setreg( NV04_PRAMDAC_GENERAL_CONTROL , lrv ) ;

  lrv = 0 ;
  setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_SA_HI_INDEX , lrv >> 8 ) ;
  setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_SA_LO_INDEX , lrv      ) ;
  modindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_RPC0_INDEX , 0xe0 , (( lrv >> 16) & 0x1f)) ;

  Nv04_LockDac(nv ) ;
  
}


void Nv04_setDacRegs( _Nv04Device * nv , _Nv04_VideoUser * user )
{

     
     unsigned long wv , lrv , offset , lc , hfc ;
     
     unsigned long htotal , hrs , hre , hbs , hbe , hde ;
     unsigned long vtotal , vrs , vre , vbs , vbe , vde ;    
     
     hde    = user->ViewX / 8 ;
     vde    = user->ViewY ;
 
    
     htotal = ( user->Timing.PixelClock / user->Timing.HSync ) ;
     hrs    =  ((user->Timing.PixelClock / user->Timing.HSync) * user->Timing.HSyncStart ) / 8 ;
     hre    =  hrs + ( user->Timing.PixelClock * user->Timing.HSyncTime ) / 8 ;
     hbs    = hde ;
     hbe    = htotal /8 ;
 
     offset = 0x1000 / 8 ;

     vtotal = ( user->Timing.HSync / user->Timing.VSync ) ;
     vrs    = ((user->Timing.HSync / user->Timing.VSync ) * user->Timing.VSyncStart ) ;
     vre    = vrs + ( user->Timing.HSync * (user->Timing.VSyncTime ) ) ;
     vbs    = vde ;
     vbe    = vtotal ;

    
     lc  = vde   ;
     hfc = 0x1ff ;

    printf("Nv04: PixelClock = %f\n",user->Timing.PixelClock) ;
    
    printf("Nv04: htotal = %6.6ld hde = %6.6ld hrs = %6.6ld hre = %6.6ld  ( hbs = %6.6ld hbe = %6.6ld )\n",htotal,hde * 8,hrs * 8,hre * 8,hbs * 8,hbe * 8)  ;
    printf("Nv04: vtotal = %6.6ld vde = %6.6ld vrs = %6.6ld vre = %6.6ld  ( vbs = %6.6ld vbe = %6.6ld )\n",vtotal,vde,vrs,vre,vbs,vbe) ;
    
    printf("offset = 0x%8.8lX lc = 0x%6.6ld 0x%8.8lX\n",offset,lc,hfc) ; 

  



     setreg8( NV04_PRMVIO_MISC__WRITE , 0x2b ) ;
     
     modindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_VRE_INDEX , 0x7f , 0x00 ) ;

     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_HDT_INDEX , ((htotal / 8) - 5) & 0xff) ; 
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_HDE_INDEX ,  (hde - 1)         & 0xff) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_HBS_INDEX ,  (hbs - 1)         & 0xff) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_HBE_INDEX , (((vbe - 1) & 0x1f) | 0x80) & 0xff) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_HRS_INDEX ,  (hrs + 2)         & 0xff) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_HRE_INDEX , ((((vbe - 1)     & 0x0020) << (7-5)) | ((hre + 2) & 0x1f)) & 0xff) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_VDT_INDEX , ((vtotal -2) & 0xff)) ;
     
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_OVL_INDEX , (  
									 ((vrs          & 0x0200) >> (9-7)) |
									 (((vde    - 1) & 0x0200) >> (9-6)) |
									 (((vtotal - 2) & 0x0200) >> (9-5)) |
									 (((lc )        & 0x0100) >> (8-4)) |
									 (((vbs - 1)    & 0x0100) >> (8-3)) |
									 (((vrs )       & 0x0100) >> (8-2)) |
									 (((vde - 1)    & 0x0100) >> (8-1)) |
									 (((vtotal - 2) & 0x0100) >> (8-0))
									 ) & 0xff 
		  ) ;
     
     
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_CELL_HT_INDEX , ( (( (lc) & 0x0200) >> (9-6)) | (((vbs - 1) & 0x0200) >> (9-5))) & 0xff) ;
     
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_VRS_INDEX , ( vrs  & 0xff )) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_VRE_INDEX , ( vre  & 0x0f )) ;     
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_VDE_INDEX , ( vde - 1 ) & 0xff ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_VBS_INDEX , ( vbs - 1 ) & 0xff ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_VBE_INDEX , ( vbe - 1 ) & 0xff ) ;

     modindexreg( nv , NV04_PRMCIO_CRX__COLOR ,0x19 , 0x1f , ((offset & 0x0700) >> ( 8 - 5))) ;  

     wv = 0 ;

     if ( user->Timing.HSyncTime == 0 ) wv |= 0x80 ;
     if ( user->Timing.VSyncTime == 0 ) wv |= 0x40 ;

     wv |= ((user->ViewX < 1280) & (user->ViewY < 1024)) ? 0x04 : 0x00 ;
     wv |= 0x38 ;

     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x1a , wv   ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x1d , 0x01 ) ;
     
     wv  = getindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x1c ) ;
    
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x1c , wv  ) ;  /* dont ask me ? */ 
     
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x1d , 0x00 ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x1e , 0x00 ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x20 , 0x10 ) ;
     
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x25 , ( 
							(( offset       & 0x0800 ) >> ( 11 - 5 )) | /* not in my ref file */
							((( hbe - 1)    & 0x0040 ) >> (  6 - 4 )) |
							((( vbs - 1)    & 0x0400 ) >> ( 10 - 3 )) |
							((( vrs)        & 0x0400 ) >> ( 10 - 2 )) |
							((( vde - 1)    & 0x0400 ) >> ( 10 - 1 )) |
							((( vtotal - 2) & 0x0400 ) >> ( 10 - 0 )) 
							) ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , 0x2d , (
							((( hfc )            & 0x0100 ) >> (  8 - 4 )) |
							((( hrs + 2)         & 0x0100 ) >> (  8 - 3 )) |
							((( hbs - 1)         & 0x0100 ) >> (  8 - 2 )) |
							((( hde - 1)         & 0x0100 ) >> (  8 - 1 )) |
							((( htotal /8 ) - 5) & 0x0100 ) >> (  8 - 0 )) 
		  ) ;
     
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_PAGE_OVFL__INDEX , 0x00 ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_OSCOL__INDEX     , 0x00 ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_HCUR_ADDR0_INDEX , 0x00 ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_HCUR_ADDR1_INDEX , 0x00 ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_VID_END0__INDEX  , 0x00 ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_LCD__INDEX       , 0x00 ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_RL0__INDEX       , 0x00 ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CRE_HCUR_ADDR0_INDEX , ( hfc    & 0x1f ) ) ;
     setindexreg( nv , NV04_PRMCIO_CRX__COLOR , NV04_CIO_CR_OFFSET_INDEX      , ( offset & 0xff ) ) ;
  
     lrv = getreg( NV_PRAMDAC_PLL_COEFF_SELECT ) ; setreg( NV_PRAMDAC_PLL_COEFF_SELECT , 0x10000700 ) ;
}

void Nv04_LoadDefaultATCAndSave( _Nv04Device * nv , unsigned char *pal )
{
  unsigned long i ;

  getreg8( NV04_PRMCIO_INP0__COLOR ) ; /* reset flop */ 
  for ( i = 0 ; i < 0x20 ; i++) 
    {     
      setreg8( NV04_PRMCIO_ARX , i ) ;
      setreg8( NV04_PRMCIO_ARX , *pal++ ) ;
    }
  getreg8( NV04_PRMCIO_INP0__COLOR ) ; /* reset flop */ 
}

void Nv04_dacLoadPalette( _Nv04Device * nv , unsigned long *xrgbp )
{
  unsigned long i , xrgb ;

  Nv04_UnlockDac(nv) ;

  Nv04_LoadDefaultATCAndSave( nv , atc_tbl ) ;

  getreg8( NV04_PRMCIO_INP0__COLOR ) ; /* reset flop */ 
  
  setreg8( NV04_PRMCIO_ARX                   , 0x14 ) ;
  setreg8( NV04_PRMCIO_ARX                   , 0x00 ) ;
  setreg8( NV04_USER_DAC_PIXEL_MASK          , 0xff ) ; 
  setreg8( NV04_USER_DAC_WRITE_MODE_ADDRESS  , 0x00 ) ; 

  for ( i = 0 ; i <= 0xff ; i++ ) 
    {
      xrgb = *xrgbp++ ;
      setreg8( NV04_USER_DAC_PALETTE_DATA      , xrgb >> 16 ) ;
      setreg8( NV04_USER_DAC_PALETTE_DATA      , xrgb >> 8  ) ;
      setreg8( NV04_USER_DAC_PALETTE_DATA      , xrgb       ) ;
    }
    
  Nv04_LockDac(nv) ;
}
		  
void Nv04_dacStartDac( _Nv04Device *nv , _Nv04_VideoUser * user)
{
  Nv04_UnlockDac(nv) ;
  setindexreg( nv , NV04_PRMVIO_SRX , NV04_PRMVIO_SR_CLOCK_INDEX , 0x20 | getindexreg( nv , NV04_PRMVIO_SRX , NV04_PRMVIO_SR_CLOCK_INDEX )) ;   
  Nv04_setDacRegs(nv,user);	
  modindexreg(  nv , NV04_PRMVIO_SRX , NV04_PRMVIO_SR_CLOCK_INDEX , ~0x20 , ( (Nv04_dacEnableImageScan(nv) == 0) << 5)) ; 
  Nv04_LockDac(nv) ;                 
}
							
							
							
