#define Nv04_CtxDma_NVM    0x00000000 /* on card mem */ 
#define Nv04_CtxDma_System 0x00000001 /* pc mem      */

_MemInstance * Nv04_MakeCtxDmaInstance( _Nv04Device   *nv       ,
					unsigned long  PhysAddr ,
					unsigned long  Limit    ,
					unsigned long  Type     
					)
{
  _MemInstance  *CtxDmaInst ;

  CtxDmaInst = Nv04_AddToInstMem( nv , NULL , 0x00000010 , "ctxDmaInst") ;
  
  if (Type == Nv04_CtxDma_NVM) setreg ( (0 ? NV04_PRAMIN) + CtxDmaInst->base + 0x00000000 , 0x00013000  ) ;
  else
    {
      switch(nv->DmaMode)
	{
	case Nv04_DMA_BUS_PCI: setreg( (0 ? NV04_PRAMIN) + CtxDmaInst->base + 0x00000000 , 0x00023000  ) ; break ;
	case Nv04_DMA_BUS_AGP: setreg( (0 ? NV04_PRAMIN) + CtxDmaInst->base + 0x00000000 , 0x00033000  ) ; break ;
	default: printf("Nv04: MakeDmaInstance ERROR unknow bus type 0x%8.8lX\n",nv->DmaMode) ; return(0) ;
	}
    }
  setreg ( (0 ? NV04_PRAMIN) + CtxDmaInst->base + 0x00000004 , Limit ) ;
  setreg ( (0 ? NV04_PRAMIN) + CtxDmaInst->base + 0x00000008 , 0x00000002  ) ;
  setreg ( (0 ? NV04_PRAMIN) + CtxDmaInst->base + 0x0000000c , 0x00000002  ) ;
 
  return(CtxDmaInst) ;
}


_MemInstance * Nv04_MakeDmaInstance( _Nv04Device   *nv       ,
				     unsigned long  PhysAddr ,
				     unsigned long  Limit     
				     )
{
  _MemInstance  *DmaInst ;
  unsigned long  i , PtCount , size ;

  PtCount = ( Limit + 1 ) / 4096 ;
  size = 0x00000008 + ( PtCount << 2) ; if (size & 0x0f) size = (size & ~0x0f) + 0x10 ; /* keep inst mem alined */
  DmaInst = Nv04_AddToInstMem( nv , NULL , size , "dmaInst") ;

  if(!DmaInst) { printf("Nv04: MakeDmaInstance ERROR can not alloc inst mem\n") ; return(0) ; } 

  switch (nv->DmaMode)
    {
    case Nv04_DMA_BUS_PCI: setreg ( (0 ? NV04_PRAMIN) + DmaInst->base + 0x00000000 , 0x00021000  ) ; break ;
    case Nv04_DMA_BUS_AGP: setreg ( (0 ? NV04_PRAMIN) + DmaInst->base + 0x00000000 , 0x00031000  ) ; break ;
    default: printf("Nv04: MakeDmaInstance ERROR unknow bus type 0x%8.8lX\n",nv->DmaMode) ; return(0) ;
    }

  setreg ( (0 ? NV04_PRAMIN) + DmaInst->base + 0x00000004 , PtCount - 1 ) ;
  
  for ( i = 0 ; i < PtCount ; i++ ) 
    setreg( (0 ? NV04_PRAMIN) + DmaInst->base + 0x00000008 + ( i * 4) , (PhysAddr + ( i << 12)) | 0x0000 ) ;
  return(DmaInst) ;
}

void Nv04_SetupDma(_Nv04Device *nv )
{
  unsigned long data , AgpCommand ;
  printf("Nv04: setup for dma\n") ;

  Nv04_PrintReg( NV04_PBUS_PCI_NV_0  ,    getreg( NV04_PBUS_PCI_NV_0  ) , 0 , "Nv04: ") ;

  Nv04_PrintReg( NV_PEXTDEV_BOOT_0   ,    getreg( NV_PEXTDEV_BOOT_0   ) , 0 , "Nv04: ") ;


  Nv04_PrintReg( NV04_PBUS_PCI_NV_17 ,    getreg( NV04_PBUS_PCI_NV_17 ) , 0 , "Nv04: ") ;
  Nv04_PrintReg( NV04_PBUS_PCI_NV_18 ,    getreg( NV04_PBUS_PCI_NV_18 ) , 0 , "Nv04: ") ;
  
  data = getreg( NV04_PBUS_PCI_NV_18 ) ;

  AgpCommand = 0 ;

  if ((data & 0xff000000) > 0x1f000000) AgpCommand |= 0x1f000000 ;
  else
    AgpCommand |= (data & 0x1f000000) ;

  if (Nv04_DMA_TYPE_AGP) AgpCommand |= 0x00000100 ;
  if (Nv04_DMA_AGP_SBA ) AgpCommand |= 0x00000200 ;
  AgpCommand |= Nv04_DMA_AGP_RATE ;

  setreg( NV04_PBUS_PCI_NV_19 , AgpCommand ) ;
  printf("Nv04: dma setup done\n") ;
  data = getreg( NV04_PBUS_PCI_NV_19 ) ;
  Nv04_PrintReg( NV04_PBUS_PCI_NV_19 , data , 0 , "Nv04: ") ;
 
  if ( !(data & 0x00000100) )
    {
      printf("Nv04: AGP not enabled using PCI\n") ;      
      nv->DmaMode = 1 ;
    }
  else 
    nv->DmaMode = 2 ;
  
}
