
#include "AGP_PCI.h"
#include "MasterControl.h"
#include "RamDac.h"
#include "extdevs.h"

/* * FIFO */
#define NV04_PFIFO 0x00003FFF:0x00002000 /* RW */ 
#define NV04_USER  0x00FFFFFF:0x00800000 /* RW */ 

/* * Graphics Engine */ 
#define NV04_PGRAPH 0x00401FFF:0x00400000 /* RW */
 
/* * Timer */ 
#define NV04_PTIMER 0x00009FFF:0x00009000 /* RW */ 

/* * Framebuffer */ 

#define NV04_PFB           0x00100FFF:0x00100000 /* RW */ 
#define NV04_PDFB          0x03FFFFFF:0x02000000 /* RW */ 
#define NV04_PNVM          0x03FFFFFF:0x02000000 /* RW */ 
#define NV04_PRAMIN        0x007FFFFF:0x00700000 /* RW */ 

#define NV04_PFB_BOOT_0                                0x00100000 /* RW */ 
#define NV04_PFB_BOOT_0_RAM_AMOUNT                1:0             /* RW */ 
#define NV04_PFB_BOOT_0_RAM_AMOUNT_32MB                0x00000000 /* RW */ 
#define NV04_PFB_BOOT_0_RAM_AMOUNT_4MB                 0x00000001 /* RW */ 
#define NV04_PFB_BOOT_0_RAM_AMOUNT_8MB                 0x00000002 /* RW */ 
#define NV04_PFB_BOOT_0_RAM_AMOUNT_16MB                0x00000003 /* RW */ 
#define NV04_PFB_BOOT_0_RAM_WIDTH_128             2:2             /* RW */ 
#define NV04_PFB_BOOT_0_RAM_WIDTH_128_OFF              0x00000000 /* RW */ 
#define NV04_PFB_BOOT_0_RAM_WIDTH_128_ON               0x00000001 /* RW */ 
#define NV04_PFB_BOOT_0_RAM_TYPE                  5:3             /* RW */ 
#define NV04_PFB_BOOT_0_RAM_TYPE_SGRAM_8MBIT           0x00000000 /* R- */ 
#define NV04_PFB_BOOT_0_RAM_TYPE_SGRAM_16MBIT          0x00000001 /* R- */ 
#define NV04_PFB_BOOT_0_RAM_TYPE_SGRAM_16MBIT_4BANK    0x00000002 /* R- */ 
#define NV04_PFB_BOOT_0_RAM_TYPE_SDRAM_16MBIT          0x00000003 /* R- */ 
#define NV04_PFB_BOOT_0_RAM_TYPE_SDRAM_64MBIT          0x00000004 /* R- */ 
#define NV04_PFB_BOOT_0_RAM_TYPE_SDRAM_64MBITX16       0x00000005 /* R- */ 
#define NV04_PFB_BOOT_0_UMA                       8:8             /* RW */ 
#define NV04_PFB_BOOT_0_UMA_DISABLE                    0x00000000 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_ENABLE                     0x00000001 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE                 15:12            /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_DEFAULT               0x00000007 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_2M                    0x00000000 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_4M                    0x00000001 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_6M                    0x00000002 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_8M                    0x00000003 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_10M                   0x00000004 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_12M                   0x00000005 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_14M                   0x00000006 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_16M                   0x00000007 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_18M                   0x00000008 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_20M                   0x00000009 /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_22M                   0x0000000a /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_24M                   0x0000000b /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_26M                   0x0000000c /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_28M                   0x0000000d /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_30M                   0x0000000e /* RW */ 
#define NV04_PFB_BOOT_0_UMA_SIZE_32M                   0x0000000f /* RW */ 

#define NV04_PFB_DEBUG_0                               0x00100080 /* RW */ 

#define NV04_PFB_CONFIG_0                              0x00100200 /* RW */
#define NV04_PFB_CONFIG_0_TYPE                   14:0             /* RW */ 
#define NV04_PFB_CONFIG_0_TYPE_OLD1024_FIXED_8BPP      0x00000120 /* RW */ 
#define NV04_PFB_CONFIG_0_TYPE_OLD1024_FIXED_16BPP     0x00000220 /* RW */ 
#define NV04_PFB_CONFIG_0_TYPE_OLD1024_FIXED_32BPP     0x00000320 /* RW */ 
#define NV04_PFB_CONFIG_0_TYPE_OLD1024_VAR_8BPP        0x00004120 /* RW */ 
#define NV04_PFB_CONFIG_0_TYPE_OLD1024_VAR_16BPP       0x00004220 /* RW */ 
#define NV04_PFB_CONFIG_0_TYPE_OLD1024_VAR_32BPP       0x00004320 /* RW */ 

#define NV04_PFB_CONFIG_1                              0x00100204 /* RW */ 
#define NV04_PFB_CONFIG_1_CAS_LATENCY            2:0              /* RW */ 
#define NV04_PFB_CONFIG_1_CAS_LATENCY_3                0x00000003 /* RW */ 
#define NV04_PFB_CONFIG_1_CAS_LATENCY_2                0x00000002 /* RW */ 
#define NV04_PFB_CONFIG_1_CAS_LATENCY_4                0x00000004 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_RAS                7:4              /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_RAS_DEFAULT              0x00000009 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_RAS_9CYCLES              0x00000008 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_RAS_8CYCLES              0x00000007 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_RAS_7CYCLES              0x00000006 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_PCHG              10:8              /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_PCHG_DEFAULT             0x00000002 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_PCHG_2CYCLES             0x00000001 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_LOW               14:12             /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_LOW_DEFAULT              0x00000006 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_LOW_7CYCLES              0x00000007 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_LOW_5CYCLES              0x00000005 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_LOW_4CYCLES              0x00000004 /* RW */ 
#define NV04_PFB_CONFIG_1_MRS_TO_RAS            18:16             /* RW */ 
#define NV04_PFB_CONFIG_1_MRS_TO_RAS_DEFAULT           0x00000001 /* RW */ 
#define NV04_PFB_CONFIG_1_MRS_TO_RAS_2CYCLES           0x00000002 /* RW */ 
#define NV04_PFB_CONFIG_1_MRS_TO_RAS_0CYCLES           0x00000000 /* RW */ 
#define NV04_PFB_CONFIG_1_WRITE_TO_READ         22:20             /* RW */ 
#define NV04_PFB_CONFIG_1_WRITE_TO_READ_DEFAULT        0x00000000 /* RW */ 
#define NV04_PFB_CONFIG_1_SPECIAL32             20:20             /* RW */ 
#define NV04_PFB_CONFIG_1_SPECIAL32_ON                 0x00000001 /* RW */ 
#define NV04_PFB_CONFIG_1_SPECIAL32_OFF                0x00000000 /* RW */ 
#define NV04_PFB_CONFIG_1_WRITE_TO_PCHG         21:21             /* RW */ 
#define NV04_PFB_CONFIG_1_WRITE_TO_PCHG_ON             0x00000001 /* RW */ 
#define NV04_PFB_CONFIG_1_WRITE_TO_PCHG_OFF            0x00000000 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_TO_CAS_M1         26:24             /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_TO_CAS_M1_DEFAULT        0x00000001 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_TO_CAS_M1_2CYCLES        0x00000002 /* RW */ 
#define NV04_PFB_CONFIG_1_RAS_TO_CAS_M1_0CYCLES        0x00000000 /* RW */ 
#define NV04_PFB_CONFIG_1_READ_TO_WRITE         30:28             /* RW */ 
#define NV04_PFB_CONFIG_1_READ_TO_WRITE_DEFAULT        0x00000004 /* RW */ 
#define NV04_PFB_CONFIG_1_READ_TO_WRITE_5CYCLES        0x00000005 /* RW */ 
#define NV04_PFB_CONFIG_1_READ_TO_WRITE_3CYCLES        0x00000003 /* RW */ 
#define NV04_PFB_CONFIG_1_READ_TO_WRITE_2CYCLES        0x00000002 /* RW */ 
#define NV04_PFB_CONFIG_1_READ_TO_PCHG          31:31             /* RW */ 
#define NV04_PFB_CONFIG_1_READ_TO_PCHG_ON              0x00000001 /* RW */ 
#define NV04_PFB_CONFIG_1_READ_TO_PCHG_OFF             0x00000000 /* RW */ 

#define NV04_PFB_RTL                                   0x00100300 /* RW */ 
#define NV04_PFB_RTL_H                           0:0              /* RW */ 
#define NV04_PFB_RTL_H_DEFAULT                         0x00000000 /* RW */ 
#define NV04_PFB_RTL_MC                          1:1              /* RW */ 
#define NV04_PFB_RTL_MC_DEFAULT                        0x00000000 /* RW */ 
#define NV04_PFB_RTL_V                           2:2              /* RW */ 
#define NV04_PFB_RTL_V_DEFAULT                         0x00000000 /* RW */ 
#define NV04_PFB_RTL_G                           3:3              /* RW */ 
#define NV04_PFB_RTL_G_DEFAULT                         0x00000000 /* RW */ 
#define NV04_PFB_RTL_GB                          4:4              /* RW */ 
#define NV04_PFB_RTL_GB_DEFAULT                        0x00000000 /* RW */
 
#define NV04_PNVM_DATA032(i) (0x02000000+(i)*4) /* RW */
#define NV04_PNVM_DATA032__SIZE_1 524288 /* */ 
#define NV04_PNVM_DATA032_VALUE 31:0 /* RW */ 
#define NV04_PNVM_DATA024(i) (0x02000000+((i)/2)*4+((i)%2)) /* RW */ 
#define NV04_PNVM_DATA024__SIZE_1 1048576 /* */ 
#define NV04_PNVM_DATA024_VALUE 23:0 /* RW */ 
#define NV04_PNVM_DATA016(i) (0x02000000+((i)/3)*4+((i)%3)) /* RW */ 
#define NV04_PNVM_DATA016__SIZE_1 1572864 /* */ 
#define NV04_PNVM_DATA016_VALUE 15:0 /* RW */ 
#define NV04_PNVM_DATA008(i) (0x02000000+(i)) /* RW */ 
#define NV04_PNVM_DATA008__SIZE_1 2097152 /* */ 
#define NV04_PNVM_DATA008_VALUE 7:0 /* RW */ 

#define NV04_PRAMIN_DATA032(i)                       (0x00700000+(i)*4) /* RW */ 
#define NV04_PRAMIN_DATA032__SIZE_1                            524288   /* */ 
#define NV04_PRAMIN_DATA032_VALUE                31:0                   /* RW */ 

#define NV04_PRAMIN_DATA024(i)                       (0x00700000+((i)/2)*4+((i)%2)) /* RW */ 
#define NV04_PRAMIN_DATA024__SIZE_1 1048576 /* */ 
#define NV04_PRAMIN_DATA024_VALUE 23:0 /* RW */ 

#define NV04_PRAMIN_DATA016(i)                       (0x00700000+((i)/3)*4+((i)%3)) /* RW */ 
#define NV04_PRAMIN_DATA016__SIZE_1 1572864 /* */ 
#define NV04_PRAMIN_DATA016_VALUE 15:0 /* RW */ 

#define NV04_PRAMIN_DATA008(i)                       (0x00700000+(i)) /* RW */ 
#define NV04_PRAMIN_DATA008__SIZE_1 2097152 /* */ 
#define NV04_PRAMIN_DATA008_VALUE 7:0 /* RW */ 

#define NV04_PDFB_DATA032(i) (0x01000000+(i)*4) /* RW */ 
#define NV04_PDFB_DATA032__SIZE_1 524288 /* */ 
#define NV04_PDFB_DATA032_VALUE 31:0 /* RW */ 
#define NV04_PDFB_DATA024(i) (0x01000000+((i)/2)*4+((i)%2)) /* RW */ 
#define NV04_PDFB_DATA024__SIZE_1 1048576 /* */ 
#define NV04_PDFB_DATA024_VALUE 23:0 /* RW */ 
#define NV04_PDFB_DATA016(i) (0x01000000+((i)/3)*4+((i)%3)) /* RW */ 
#define NV04_PDFB_DATA016__SIZE_1 1572864 /* */ 
#define NV04_PDFB_DATA016_VALUE 15:0 /* RW */ 
#define NV04_PDFB_DATA008(i) (0x01000000+(i)) /* RW */ 
#define NV04_PDFB_DATA008__SIZE_1 2097152 /* */ 
#define NV04_PDFB_DATA008_VALUE 7:0 /* RW */ 

#define NV04_PFB_SCRAMBLE(i) (0x00100400+((i)*4)) /* RW */ 

#define NV04_PFB_FIFO_CTL                           0x00100208 /* RW */ 
#define NV04_PFB_FIFO_CTL_TEST               0:0               /* RW */ 
#define NV04_PFB_FIFO_CTL_TEST_DISABLE              0x00000000 /* RW */ 
#define NV04_PFB_FIFO_CTL_TEST_ENABLE               0x00000001 /* RW */ 
#define NV04_PFB_FIFO_CTL_MEM_SEL            7:4               /* RW */
#define NV04_PFB_FIFO_CTL_MEM_SEL_RT0_REQ           0x00000000 /* RW */ 
#define NV04_PFB_FIFO_CTL_MEM_SEL_RT1_REQ           0x00000001 /* RW */ 
#define NV04_PFB_FIFO_CTL_MEM_SEL_RT2_REQ           0x00000002 /* RW */ 
#define NV04_PFB_FIFO_CTL_MEM_SEL_NRT_REQ           0x00000003 /* RW */ 
#define NV04_PFB_FIFO_CTL_MEM_SEL_RT_DATA           0x00000004 /* RW */ 
#define NV04_PFB_FIFO_CTL_MEM_SEL_NRT_DATA          0x00000005 /* RW */ 
#define NV04_PFB_FIFO_CTL_MEM_SEL_RD_DATA           0x00000006 /* RW */ 
#define NV04_PFB_FIFO_CTL_BIT_SEL           11:8               /* RW */ 
#define NV04_PFB_FIFO_CTL_BIT_SEL_31_0              0x00000000 /* RW */ 
#define NV04_PFB_FIFO_CTL_BIT_SEL_63_32             0x00000001 /* RW */ 
#define NV04_PFB_FIFO_CTL_BIT_SEL_95_64             0x00000002 /* RW */ 
#define NV04_PFB_FIFO_CTL_BIT_SEL_127_96            0x00000003 /* RW */ 
#define NV04_PFB_FIFO_CTL_BIT_SEL_159_128           0x00000004 /* RW */
#define NV04_PFB_FIFO_CTL_ADDR_SEL          15:12              /* RW */ 
#define NV04_PFB_FIFO_DATA                          0x0010020c /* RW */ 

/* * VGA */ 
#define NV04_PCRTC   0x00600FFF:0x00600000 /* RW */ 
#define NV04_PRMCIO  0x00601FFF:0x00601000 /* RW */ 
#define NV04_PRMVIO  0x000C7FFF:0x000C0000 /* RW */ 
#define NV04_PRMVGA  0x000BFFFF:0x000A0000 /* RW */ 
#define NV04_CIO          0x3DF:0x3B0 /* -- */ 

#define NV04_PCRTC_INTR_0                           0x00600100 /* RW */ 
#define NV04_PCRTC_INTR_0_VBLANK             0:0               /* RW */ 
#define NV04_PCRTC_INTR_0_VBLANK_NOT_PENDING        0x00000000 /* R- */ 
#define NV04_PCRTC_INTR_0_VBLANK_PENDING            0x00000001 /* R- */ 
#define NV04_PCRTC_INTR_0_VBLANK_RESET              0x00000001 /* -W */ 
#define NV04_PCRTC_INTR_EN_0                        0x00600140 /* RW */ 
#define NV04_PCRTC_INTR_EN_0_VBLANK          0:0               /* RW */ 
#define NV04_PCRTC_INTR_EN_0_VBLANK_DISABLED        0x00000000 /* RW */ 
#define NV04_PCRTC_INTR_EN_0_VBLANK_ENABLED         0x00000001 /* RW */ 

#define NV04_PCRTC_START                            0x00600800 /* RW */ 
#define NV04_PCRTC_START_ADDRESS            24:2               /* RW */ 

#define NV04_PCRTC_CONFIG                           0x00600804 /* RW */ 
#define NV04_PCRTC_CONFIG_START_ADDRESS      2:0               /* RW */ 
#define NV04_PCRTC_CONFIG_START_ADDRESS_VGA         0x00000000 /* RW */ 
#define NV04_PCRTC_CONFIG_START_ADDRESS_NON_VGA     0x00000001 /* RW */ 
#define NV04_PCRTC_CONFIG_START_ADDRESS_HSYNC       0x00000004 /* RW */ 

#define NV04_PCRTC_RASTER                           0x00600808 /* R- */ 
#define NV04_PCRTC_RASTER_POSITION          10:0               /* R- */ 
#define NV04_PCRTC_RASTER_SA_LOAD           13:12              /* R- */ 
#define NV04_PCRTC_RASTER_SA_LOAD_DISPLAY           0x00000000 /* R- */ 
#define NV04_PCRTC_RASTER_SA_LOAD_BEFORE            0x00000001 /* R- */ 
#define NV04_PCRTC_RASTER_SA_LOAD_AFTER             0x00000002 /* R- */ 
#define NV04_PCRTC_RASTER_VERT_BLANK        16:16              /* R- */ 
#define NV04_PCRTC_RASTER_VERT_BLANK_ACTIVE         0x00000001 /* R- */ 
#define NV04_PCRTC_RASTER_VERT_BLANK_INACTIVE       0x00000000 /* R- */ 
#define NV04_PCRTC_FIFO_CNTRL                       0x00600810 /* RW */ 
#define NV04_PCRTC_FIFO_CNTRL_ADDRESS        4:0               /* RW */ 
#define NV04_PCRTC_FIFO_CNTRL_RAM            9:8               /* RW */ 
#define NV04_PCRTC_FIFO_CNTRL_TESTMODE      16:16              /* RW */ 
#define NV04_PCRTC_FIFO_CNTRL_TESTMODE_ENABLE        1         /* RW */ 
#define NV04_PCRTC_FIFO_CNTRL_TESTMODE_DISABLE       0         /* RW */ 

#define NV04_PCRTC_FIFO_DATA                        0x00600814 /* RW */ 
#define NV04_PCRTC_FIFO_DATA_VAL            31:0               /* RW */ 

#define NV04_PRMVIO_MBEN                            0x000C0094 /* RW */ 
#define NV04_VIO_MBEN                               0x00000094 /* RW */ 
#define NV04_PRMVIO_ADDEN                           0x000C46e8 /* RW */ 
#define NV04_VIO_ADDEN                              0x000046e8 /* RW */ 
#define NV04_PRMVIO_VSE1                            0x000C0102 /* RW */ 
#define NV04_VIO_VSE1                               0x00000102 /* RW */ 
#define NV04_PRMVIO_VSE2                            0x000C03c3 /* RW */ 
#define NV04_VIO_VSE2                               0x000003c3 /* RW */ 

#define NV04_PRMVIO_MISC__READ                      0x000C03cc /* R- */ 
#define NV04_VIO_MISC__READ                         0x000003cc /* R- */ 
#define NV04_PRMVIO_MISC__WRITE                     0x000C03c2 /* -W */ 
#define NV04_VIO_MISC__WRITE                        0x000003c2 /* -W */ 

#define NV04_PRMCIO_INP0                            0x006013c2 /* R- */ 
#define NV04_CIO_INP0                               0x000003c2 /* R- */ 
#define NV04_PRMCIO_INP0__MONO                      0x006013ba /* R- */ 
#define NV04_CIO_INP0__MONO                         0x000003ba /* R- */ 

#define NV04_PRMCIO_INP0__COLOR                     0x006013da /* R- */ 
#define NV04_CIO_INP0__COLOR                        0x000003da /* R- */ 

#define NV04_PRMCIO_INP0__READ_MONO                 0x006013ca /* R- */ 
#define NV04_CIO_INP0__READ_MONO                    0x000003ca /* R- */ 

#define NV04_PRMCIO_INP0__WRITE_MONO                0x006013ba /* -W */ 
#define NV04_CIO_INP0__WRITE_MONO                   0x000003ba /* -W */ 

#define NV04_PRMCIO_INP0__WRITE_COLOR               0x006013da /* -W */ 
#define NV04_CIO_INP0__WRITE_COLOR                  0x000003da /* -W */ 

#define NV04_PRMVIO_SRX                             0x000C03c4 /* RW */ 

#define NV04_VIO_SRX                                0x000003c4 /* RW */ 

#define NV04_PRMVIO_SR_RESET                        0x000C03c5 /* RW */ 

#define NV04_PRMVIO_SR_RESET_INDEX                  0x00000000 /* */ 

#define NV04_VIO_SR_RESET                           0x000003c5 /* RW */ 

#define NV04_VIO_SR_RESET_INDEX                     0x00000000 /* */ 
#define NV04_PRMVIO_SR_CLOCK                        0x000C03c5 /* RW */ 
#define NV04_PRMVIO_SR_CLOCK_INDEX                  0x00000001 /* */ 
#define NV04_VIO_SR_CLOCK                           0x000003c5 /* RW */ 
#define NV04_VIO_SR_CLOCK_INDEX                     0x00000001 /* */ 
#define NV04_PRMVIO_SR_PLANE_MASK                   0x000C03c5 /* RW */ 
#define NV04_PRMVIO_SR_PLANE_MASK_INDEX             0x00000002 /* */ 
#define NV04_VIO_SR_PLANE_MASK                      0x000003c5 /* RW */ 
#define NV04_VIO_SR_PLANE_MASK_INDEX                0x00000002 /* */ 
#define NV04_PRMVIO_SR_CHAR_MAP                     0x000C03c5 /* RW */ 
#define NV04_PRMVIO_SR_CHAR_MAP_INDEX               0x00000003 /* */ 
#define NV04_VIO_SR_CHAR_MAP                        0x000003c5 /* RW */ 
#define NV04_VIO_SR_CHAR_MAP_INDEX                  0x00000003 /* */ 
#define NV04_PRMVIO_SR_MEM_MODE                     0x000C03c5 /* RW */ 
#define NV04_PRMVIO_SR_MEM_MODE_INDEX               0x00000004 /* */ 
#define NV04_VIO_SR_MEM_MODE                        0x000003c5 /* RW */ 
#define NV04_VIO_SR_MEM_MODE_INDEX                  0x00000004 /* */ 
#define NV04_PRMVIO_GRX                             0x000C03ce /* RW */ 
#define NV04_VIO_GRX                                0x000003ce /* RW */ 
#define NV04_PRMVIO_GX_SR                           0x000C03cf /* RW */ 
#define NV04_PRMVIO_GX_SR_INDEX                     0x00000000 /* */ 
#define NV04_VIO_GX_SR                              0x000003cf /* RW */ 
#define NV04_VIO_GX_SR_INDEX                        0x00000000 /* */ 
#define NV04_PRMVIO_GX_SREN                         0x000C03cf /* RW */ 
#define NV04_PRMVIO_GX_SREN_INDEX                   0x00000001 /* */ 
#define NV04_VIO_GX_SREN                            0x000003cf /* RW */ 
#define NV04_VIO_GX_SREN_INDEX                      0x00000001 /* */ 
#define NV04_PRMVIO_GX_CCOMP                        0x000C03cf /* RW */ 
#define NV04_PRMVIO_GX_CCOMP_INDEX                  0x00000002 /* */ 
#define NV04_VIO_GX_CCOMP                           0x000003cf /* RW */ 
#define NV04_VIO_GX_CCOMP_INDEX                     0x00000002 /* */ 
#define NV04_PRMVIO_GX_ROP                          0x000C03cf /* RW */ 
#define NV04_PRMVIO_GX_ROP_INDEX                    0x00000003 /* */ 
#define NV04_VIO_GX_ROP                             0x000003cf /* RW */ 
#define NV04_VIO_GX_ROP_INDEX                       0x00000003 /* */ 
#define NV04_PRMVIO_GX_READ_MAP                     0x000C03cf /* RW */ 
#define NV04_PRMVIO_GX_READ_MAP_INDEX               0x00000004 /* */ 
#define NV04_VIO_GX_READ_MAP                        0x000003cf /* RW */ 
#define NV04_VIO_GX_READ_MAP_INDEX                  0x00000004 /* */ 
#define NV04_PRMVIO_GX_MODE                         0x000C03cf /* RW */
#define NV04_PRMVIO_GX_MODE_INDEX                   0x00000005 /* */ 
#define NV04_VIO_GX_MODE                            0x000003cf /* RW */ 
#define NV04_VIO_GX_MODE_INDEX                      0x00000005 /* */ 
#define NV04_PRMVIO_GX_MISC                         0x000C03cf /* RW */ 
#define NV04_PRMVIO_GX_MISC_INDEX                   0x00000006 /* */ 
#define NV04_VIO_GX_MISC                            0x000003cf /* RW */ 
#define NV04_VIO_GX_MISC_INDEX                      0x00000006 /* */ 
#define NV04_PRMVIO_GX_DONT_CARE                    0x000C03cf /* RW */ 
#define NV04_PRMVIO_GX_DONT_CARE_INDEX              0x00000007 /* */ 
#define NV04_VIO_GX_DONT_CARE                       0x000003cf /* RW */ 
#define NV04_VIO_GX_DONT_CARE_INDEX                 0x00000007 /* */ 
#define NV04_PRMVIO_GX_BIT_MASK                     0x000C03cf /* RW */ 
#define NV04_PRMVIO_GX_BIT_MASK_INDEX               0x00000008 /* */ 
#define NV04_VIO_GX_BIT_MASK                        0x000003cf /* RW */ 
#define NV04_VIO_GX_BIT_MASK_INDEX                  0x00000008 /* */ 

#define NV04_PRMCIO_ARX                             0x006013c0 /* RW */ 

#define NV04_CIO_ARX                                0x000003c0 /* RW */ 

#define NV04_PRMCIO_AR_PALETTE__WRITE               0x006013c0 /* -W */ 
#define NV04_CIO_AR_PALETTE__WRITE                  0x000003c0 /* -W */ 
#define NV04_PRMCIO_AR_PALETTE__READ                0x006013c1 /* R- */ 
#define NV04_CIO_AR_PALETTE__READ                   0x000003c1 /* R- */ 

#define NV04_PRMCIO_AR_MODE__WRITE                  0x006013c0 /* -W */ 
#define NV04_PRMCIO_AR_MODE__READ                   0x006013c1 /* R- */ 
#define NV04_PRMCIO_AR_MODE_INDEX                   0x00000010 /* */ 
#define NV04_CIO_AR_MODE__WRITE                     0x000003c0 /* -W */ 
#define NV04_CIO_AR_MODE__READ                      0x000003c1 /* R- */ 
#define NV04_CIO_AR_MODE_INDEX                      0x00000010 /* */

#define NV04_PRMCIO_AR_OSCAN__WRITE                 0x006013c0 /* -W */ 
#define NV04_PRMCIO_AR_OSCAN__READ                  0x006013c1 /* R- */ 
#define NV04_PRMCIO_AR_OSCAN_INDEX                  0x00000011 /* */ 

#define NV04_CIO_AR_OSCAN__WRITE                    0x000003c0 /* -W */ 
#define NV04_CIO_AR_OSCAN__READ                     0x000003c1 /* R- */ 
#define NV04_CIO_AR_OSCAN_INDEX                     0x00000011 /* */ 
#define NV04_PRMCIO_AR_PLANE__WRITE                 0x006013c0 /* -W */ 
#define NV04_PRMCIO_AR_PLANE__READ                  0x006013c1 /* R- */ 
#define NV04_PRMCIO_AR_PLANE_INDEX                  0x00000012 /* */ 
#define NV04_CIO_AR_PLANE__WRITE                    0x000003c0 /* -W */ 
#define NV04_CIO_AR_PLANE__READ                     0x000003c1 /* R- */ 
#define NV04_CIO_AR_PLANE_INDEX                     0x00000012 /* */ 
#define NV04_PRMCIO_AR_HPP__WRITE                   0x006013c0 /* -W */ 
#define NV04_PRMCIO_AR_HPP__READ                    0x006013c1 /* R- */ 
#define NV04_PRMCIO_AR_HPP_INDEX                    0x00000013 /* */ 
#define NV04_CIO_AR_HPP__WRITE                      0x000003c0 /* -W */ 
#define NV04_CIO_AR_HPP__READ                       0x000003c1 /* R- */ 
#define NV04_CIO_AR_HPP_INDEX                       0x00000013 /* */ 
#define NV04_PRMCIO_AR_CSEL__WRITE                  0x006013c0 /* -W */ 
#define NV04_PRMCIO_AR_CSEL__READ                   0x006013c1 /* R- */ 
#define NV04_PRMCIO_AR_CSEL_INDEX                   0x00000014 /* */ 
#define NV04_CIO_AR_CSEL__WRITE                     0x000003c0 /* -W */ 
#define NV04_CIO_AR_CSEL__READ                      0x000003c1 /* R- */ 
#define NV04_CIO_AR_CSEL_INDEX                      0x00000014 /* */ 

#define NV04_PRMCIO_CRX__MONO                       0x006013b4 /* RW */ 
#define NV04_PRMCIO_CRX__COLOR                      0x006013d4 /* RW */ 

#define NV04_CIO_CRX__MONO                          0x000003b4 /* RW */ 
#define NV04_CIO_CRX__COLOR                         0x000003d4 /* RW */
 
#define NV04_PRMCIO_CR__MONO                        0x006013b5 /* RW */ 
#define NV04_PRMCIO_CR__COLOR                       0x006013d5 /* RW */ 

#define NV04_CIO_CR__MONO                           0x000003b5 /* RW */ 
#define NV04_CIO_CR__COLOR                          0x000003d5 /* RW */
 
#define NV04_CIO_CR_HDT_INDEX                       0x00000000 /* */ 
#define NV04_CIO_CR_HDE_INDEX                       0x00000001 /* */ 
#define NV04_CIO_CR_HBS_INDEX                       0x00000002 /* */ 
#define NV04_CIO_CR_HBE_INDEX                       0x00000003 /* */ 
#define NV04_CIO_CR_HBE_4_0                   4:0              /* RW */ 
#define NV04_CIO_CR_HRS_INDEX                       0x00000004 /* */ 
#define NV04_CIO_CR_HRE_INDEX                       0x00000005 /* */ 
#define NV04_CIO_CR_HRE_HBE_5                 7:7              /* RW */ 
#define NV04_CIO_CR_HRE_4_0                   4:0              /* RW */ 
#define NV04_CIO_CR_VDT_INDEX                       0x00000006 /* */ 
#define NV04_CIO_CR_OVL_INDEX                       0x00000007 /* */ 
#define NV04_CIO_CR_OVL_VDE_8                 1:1              /* RW */ 
#define NV04_CIO_CR_OVL_VDE_9                 6:6              /* RW */ 
#define NV04_CIO_CR_OVL_VDT_8                 0:0              /* RW */ 
#define NV04_CIO_CR_OVL_VDT_9                 5:5              /* RW */ 
#define NV04_CIO_CR_OVL_VBS_8                 3:3              /* RW */ 
#define NV04_CIO_CR_OVL_VRS_8                 2:2              /* RW */ 
#define NV04_CIO_CR_OVL_VRS_9                 7:7              /* RW */ 
#define NV04_CIO_CR_RSAL_INDEX                      0x00000008 /* */ 
#define NV04_CIO_CR_RSAL_PANNING              6:5              /* RW */ 
#define NV04_CIO_CR_CELL_HT_INDEX                   0x00000009 /* */ 
#define NV04_CIO_CR_CELL_HT_SCANDBL           7:7              /* RW */ 
#define NV04_CIO_CR_CELL_HT_VBS_9             5:5              /* RW */ 
#define NV04_CIO_CR_CURS_ST_INDEX                   0x0000000A /* */ 
#define NV04_CIO_CR_CURS_END_INDEX                  0x0000000B /* */
 
#define NV04_CIO_CR_SA_HI_INDEX                     0x0000000C /* */ 

#define NV04_CIO_CR_SA_LO_INDEX                     0x0000000D /* */ 

#define NV04_CIO_CR_TCOFF_HI_INDEX                  0x0000000E /* */ 
#define NV04_CIO_CR_TCOFF_LO_INDEX                  0x0000000F /* */ 
#define NV04_CIO_CR_VRS_INDEX                       0x00000010 /* */ 
#define NV04_CIO_CR_VRE_INDEX                       0x00000011 /* */ 
#define NV04_CIO_CR_VRE_3_0                   3:0              /* RW */ 
#define NV04_CIO_CR_VDE_INDEX                       0x00000012 /* */ 

#define NV04_CIO_CR_OFFSET_INDEX                    0x00000013 /* */ 

#define NV04_CIO_CR_ULINE_INDEX                     0x00000014 /* */ 
#define NV04_CIO_CR_VBS_INDEX                       0x00000015 /* */ 
#define NV04_CIO_CR_VBE_INDEX                       0x00000016 /* */ 
#define NV04_CIO_CR_MODE_INDEX                      0x00000017 /* */ 
#define NV04_CIO_CR_LCOMP_INDEX                     0x00000018 /* */ 
#define NV04_CIO_CR_GDATA_INDEX                     0x00000022 /* */ 
#define NV04_CIO_CR_ARFF_INDEX                      0x00000024 /* */ 
#define NV04_CIO_CR_ARX_INDEX                       0x00000026 /* */ 
#define NV04_PRMCIO_CRE__MONO                       0x006013b5 /* RW */ 
#define NV04_PRMCIO_CRE__COLOR                      0x006013d5 /* RW */ 
#define NV04_CIO_CRE__MONO                          0x000003b5 /* RW */ 
#define NV04_CIO_CRE__COLOR                         0x000003d5 /* RW */ 

#define NV04_CIO_CRE_RPC0_INDEX                     0x00000019 /* */ 
#define NV04_CIO_CRE_RPC0_START               4:0              /* RW */ 
#define NV04_CIO_CRE_RPC0_OFFSET_10_8         7:5              /* RW */ 


#define NV04_CIO_CRE_RPC1_INDEX                     0x0000001A /* */ 
#define NV04_CIO_CRE_RPC1_LARGE               2:2              /* RW */ 

#define NV04_CIO_CRE_FF_INDEX                       0x0000001B /* */ 
#define NV04_CIO_CRE_FF_BURST                 2:0              /* RW */ 
#define NV04_CIO_CRE_FF_BURST_8                     0x00000000 /* RW */ 
#define NV04_CIO_CRE_FF_BURST_32                    0x00000001 /* RW */ 
#define NV04_CIO_CRE_FF_BURST_64                    0x00000002 /* RW */ 
#define NV04_CIO_CRE_FF_BURST_128                   0x00000003 /* RW */ 
#define NV04_CIO_CRE_FF_BURST_256                   0x00000004 /* RW */ 

#define NV04_CIO_CRE_ENH_INDEX                      0x0000001C /* */ 
#define NV04_CIO_CRE_PAGE0_INDEX                    0x0000001D /* */
#define NV04_CIO_CRE_PAGE1_INDEX                    0x0000001E /* */ 

#define NV04_CIO_SR_LOCK_INDEX                      0x0000001F /* */ 
#define NV04_CIO_SR_UNLOCK_RW_VALUE                 0x00000057 /* */ 
#define NV04_CIO_SR_UNLOCK_RO_VALUE                 0x00000075 /* */ 
#define NV04_CIO_SR_LOCK_VALUE                      0x00000099 /* */ 

#define NV04_CIO_CRE_FFLWM__INDEX                   0x00000020 /* */ 
#define NV04_CIO_CRE_FFLWM_LWM                5:0              /* RW */ 

#define NV04_CIO_CRE_FABID_INDEX                    0x00000025 /* */

#define NV04_CIO_CRE_LSR_INDEX                      0x00000025 /* */ 
#define NV04_CIO_CRE_LSR_VDE_10               1:1              /* RW */ 
#define NV04_CIO_CRE_LSR_VDT_10               0:0              /* RW */ 
#define NV04_CIO_CRE_LSR_HBE_6                4:4              /* RW */ 
#define NV04_CIO_CRE_LSR_VBS_10               3:3              /* RW */ 
#define NV04_CIO_CRE_LSR_VRS_10               2:2              /* RW */ 

#define NV04_CIO_CRE_CHIP_ID_INDEX                  0x00000027 /* */ 

#define NV04_CIO_CRE_PIXEL_INDEX                    0x00000028 /* */ 
#define NV04_CIO_CRE_PIXEL_TV_ADJ             5:3              /* RW */ 
#define NV04_CIO_CRE_PIXEL_FORMAT             1:0              /* RW */ 
#define NV04_CIO_CRE_PIXEL_FORMAT_VGA               0x00000000 /* RW */ 
#define NV04_CIO_CRE_PIXEL_FORMAT_8BPP              0x00000001 /* RW */ 
#define NV04_CIO_CRE_PIXEL_FORMAT_16BPP             0x00000002 /* RW */ 
#define NV04_CIO_CRE_PIXEL_FORMAT_32BPP             0x00000003 /* RW */

 
#define NV04_CIO_CRE_PAGE_OVFL__INDEX               0x00000029 /* */ 
#define NV04_CIO_CRE_OSCOL__INDEX                   0x0000002A /* */ 
#define NV04_CIO_CRE_SCRATCH0__INDEX                0x0000002B /* */ 
#define NV04_CIO_CRE_SCRATCH1__INDEX                0x0000002C /* */ 
#define NV04_CIO_CRE_HEB__INDEX                     0x0000002D /* */
#define NV04_CIO_CRE_HEB_SA_24                6:6 /* RW */ 
#define NV04_CIO_CRE_HEB_SA_23                5:5 /* RW */ 
#define NV04_CIO_CRE_HEB_ILC_8                4:4 /* RW */ 
#define NV04_CIO_CRE_HEB_HRS_8                3:3 /* RW */ 
#define NV04_CIO_CRE_HEB_HBS_8                2:2 /* RW */ 
#define NV04_CIO_CRE_HEB_HDE_8                1:1 /* RW */ 
#define NV04_CIO_CRE_HEB_HDT_8                0:0 /* RW */ 
#define NV04_CIO_CRE_HCUR_ADDR2_INDEX               0x0000002f /* */ 
#define NV04_CIO_CRE_HCUR_ADDR0_INDEX               0x00000030 /* */ 
#define NV04_CIO_CRE_HCUR_ASI                 7:7 /* RW */ 
#define NV04_CIO_CRE_HCUR_ADDR0_ADR           6:0 /* RW */ 
#define NV04_CIO_CRE_HCUR_ADDR1_INDEX               0x00000031 /* */
#define NV04_CIO_CRE_HCUR_ADDR1_ADR           7:2 /* RW */ 
#define NV04_CIO_CRE_HCUR_ADDR1_CUR_DBL       1:1 /* RW */ 
#define NV04_CIO_CRE_HCUR_ADDR1_ENABLE        0:0 /* RW */ 
#define NV04_CIO_CRE_VID_END0__INDEX                0x00000032 /* */ 
#define NV04_CIO_CRE_LCD__INDEX                     0x00000033 /* */ 
#define NV04_CIO_CRE_LCD_LCD_SELECT           0:0 /* RW */ 
#define NV04_CIO_CRE_LCD_STEREO_ENABLE        7:7 /* RW */ 
#define NV04_CIO_CRE_RL0__INDEX                     0x00000034 /* */ 
#define NV04_CIO_CRE_RL1__INDEX                     0x00000035 /* */ 
#define NV04_CIO_CRE_DDC0_STATUS__INDEX             0x00000036 /* */ 
#define NV04_CIO_CRE_DDC0_WR__INDEX                 0x00000037 /* */ 
#define NV04_CIO_CRE_RMA__INDEX                     0x00000038 /* */ 
#define NV04_CIO_CRE_ILACE__INDEX                   0x00000039 /* */ 
#define NV04_CIO_CRE_SCRATCH2__INDEX                0x0000003A /* */ 
#define NV04_CIO_CRE_SCRATCH3__INDEX                0x0000003B /* */
#define NV04_CIO_CRE_SCRATCH4__INDEX                0x0000003C /* */ 
#define NV04_CIO_CRE_TREG__INDEX                    0x0000003D /* */ 
#define NV04_CIO_CRE_TREG_HCNT                6:6 /* RW */ 
#define NV04_CIO_CRE_TREG_VCNT                4:4 /* RW */ 
#define NV04_CIO_CRE_TREG_SHADOW              0:0 /* RW */ 
#define NV04_CIO_CRE_TREG_HCNT_INDEX                0x00000000 /* */ 
#define NV04_CIO_CRE_TREG_VCNTA_INDEX               0x00000006 /* */ 
#define NV04_CIO_CRE_TREG_VCNTB_INDEX               0x00000007 /* */ 
#define NV04_CIO_CRE_DDC_STATUS__INDEX              0x0000003E /* */ 
#define NV04_CIO_CRE_DDC_WR__INDEX                  0x0000003F /* */ 
#define NV04_CIO_CRE_PCI_TO__INDEX                  0x00000040 /* */ 
#define NV04_CIO_CRE_PCI_TO_DELAY             7:0 /* -W */ 
